package com.mcwl.web.controller.memberCenter;

import cn.hutool.core.bean.BeanUtil;
import com.mcwl.common.core.domain.R;
import com.mcwl.memberCenter.domain.Benefit;
import com.mcwl.memberCenter.domain.BenefitValue;
import com.mcwl.memberCenter.domain.dto.BenefitDto;
import com.mcwl.memberCenter.domain.vo.BenefitVo;
import com.mcwl.memberCenter.enums.MemberBenefitTypeEnum;
import com.mcwl.memberCenter.service.BenefitService;
import com.mcwl.memberCenter.service.BenefitValueService;
import com.mcwl.memberCenter.service.MemberBenefitService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/***
 * 权益
 */
@RestController
@RequestMapping("benefit")
@RequiredArgsConstructor
@Api(tags = "会员中心")
public class BenefitController {

    private final BenefitService benefitService;

    private final BenefitValueService benefitValueService;

    /**
     * 权益列表
     */
//    @PreAuthorize("@ss.hasPermi('system:benefit:list')")
    @ApiOperation(value = "权益列表")
    @GetMapping("list")
    public R<List<BenefitVo>> list() {
        List<Benefit> benefitList = benefitService.list();
        Map<Long, Double> benefitValueMap = benefitValueService.list()
                .stream()
                .collect(Collectors.toMap(BenefitValue::getId, BenefitValue::getBenefitValue));

        List<BenefitVo> benefitVoList = new ArrayList<>();
        for (Benefit benefit : benefitList) {
            BenefitVo benefitVo = BeanUtil.copyProperties(benefit, BenefitVo.class);
            Double value = benefitValueMap.get(benefit.getBenefitDiscountId());
            benefitVo.setBenefitType(benefit.getBenefitType().getValue());
            benefitVo.setBenefitDiscount(value);
            benefitVoList.add(benefitVo);
        }

        return R.ok(benefitVoList);
    }

    /**
     * 添加权益
     */
//    @PreAuthorize("@ss.hasPermi('system:benefit:add')")
    @ApiOperation(value = "添加权益")
    @PostMapping("add")
    @Transactional(rollbackFor = Exception.class)
    public R<Object> add(@RequestBody @Valid BenefitDto benefitDto) {
        Double value = benefitDto.getBenefitDiscount();
        BenefitValue benefitValue = new BenefitValue();
        benefitValue.setBenefitValue(value);
        benefitValueService.save(benefitValue);

        benefitDto.setId(null);
        Benefit benefit = BeanUtil.copyProperties(benefitDto, Benefit.class);
        benefit.setBenefitDiscountId(benefitValue.getId());
        benefitService.save(benefit);

        return R.ok();
    }

    /**
     * 权益详情
     */
//    @PreAuthorize("@ss.hasPermi('system:benefit:query')")
    @ApiOperation(value = "权益详情")
    @GetMapping("detail")
    public R<BenefitVo> detail(Long id) {
        Benefit benefit = benefitService.getById(id);
        BenefitVo benefitVo = BeanUtil.copyProperties(benefit, BenefitVo.class);
        benefitVo.setBenefitType(benefit.getBenefitType().getValue());

        BenefitValue benefitValue = benefitValueService.lambdaQuery().eq(BenefitValue::getId, benefit.getBenefitDiscountId()).one();
        benefitVo.setBenefitDiscount(benefitValue.getBenefitValue());

        return R.ok(benefitVo);
    }

    /**
     * 修改权益
     */
//    @PreAuthorize("@ss.hasPermi('system:benefit:edit')")
    @ApiOperation(value = "修改权益")
    @PostMapping("update")
    public R<Object> update(@RequestBody @Valid BenefitDto benefitDto) {

        Benefit benefit = BeanUtil.copyProperties(benefitDto, Benefit.class);
        benefitService.updateById(benefit);

        BenefitValue benefitValue = benefitValueService.lambdaQuery().eq(BenefitValue::getId, benefit.getBenefitDiscountId()).one();
        benefitValue.setBenefitValue(benefitDto.getBenefitDiscount());
        benefitValueService.updateById(benefitValue);


        return R.ok();

    }

    //    @PreAuthorize("@ss.hasPermi('system:benefit:remove')")
    @ApiOperation(value = "删除权益")
    @GetMapping("delete")
    public R<Object> delete(@NotNull(message = "id不能为空") Long id) {
        benefitService.removeById(id);
        return R.ok();
    }


}
