package com.mcwl.communityCenter.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.constant.HttpStatus;
import com.mcwl.common.core.page.PageDomain;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.obs.ObsUtils;
import com.mcwl.communityCenter.domain.CommunityFile;
import com.mcwl.communityCenter.domain.dto.*;
import com.mcwl.communityCenter.mapper.CommunityFileMapper;
import com.mcwl.communityCenter.service.CommunityFileService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;

@Service
@RequiredArgsConstructor
public class CommunityFileServiceImpl extends ServiceImpl<CommunityFileMapper, CommunityFile> implements CommunityFileService {

    private final ObsUtils obsUtils;
    @Override
    public void upload(CommunityUploadFileRes communityUploadFileRes) {
        Long tenantId = communityUploadFileRes.getTenantId();
        Long communityId = communityUploadFileRes.getCommunityId();
        MultipartFile file = communityUploadFileRes.getFile();
        Map<String, String> map = obsUtils.uploadFile(file);
        String filename = file.getOriginalFilename();
        long fileSize = file.getSize();

        CommunityFile communityFile = CommunityFile.builder()
                .tenantId(tenantId)
                .communityId(communityId)
                .userId(SecurityUtils.getUserId())
                .fileUrl(map.get("path"))
                .objectKey(map.get("objectKey"))
                .fileName(filename)
                .fileSize(fileSize)
                .build();
        baseMapper.insert(communityFile);
    }

    @Override
    public TableDataInfo getCommunityFileList(CommunityFilePageListRes communityFilePageListRes) {
        Page<CommunityFile> page = new Page<>(communityFilePageListRes.getPageNum(), communityFilePageListRes.getPageSize());
        List<CommunityFile> publishList = baseMapper.getCommunityFileList(page, communityFilePageListRes);

        // 封装返回
        TableDataInfo tableDataInfo = new TableDataInfo();
        tableDataInfo.setRows(publishList);
        tableDataInfo.setTotal(page.getTotal());
        tableDataInfo.setCode(HttpStatus.SUCCESS);
        tableDataInfo.setMsg("查询成功");
        return tableDataInfo;
    }
}
